﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectBomber
{
    internal class DrawningBomber
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityBomber EntityBomber { get; private set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// Левая координата прорисовки бомбардировщика
        /// </summary>
        private int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки бомбардировщика
        /// </summary>
        private int _startPosY;
        /// <summary>
        /// Ширина прорисовки бомбардировщика
        /// </summary>
        private int _bomberWidth = 50;
        /// <summary>
        /// Высота прорисовки бомбардировщика
        /// </summary>
        private int _bomberHeight = 110;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет крыльев</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="bombs">Признак наличия бомб</param>
        /// <param name="fuelTanks">Признак наличия топливных баков</param>
        /// <param name="line">Признак наличия полосы</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена, нельзя создать объект в этих размерах</returns>
        public bool Init(int speed, double weight, Color bodyColor, Color additionalColor, bool bombs, bool fuelTanks, bool line, int width, int height, int bomberwidth, int bomberheight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            _bomberWidth = bomberwidth;
            _bomberHeight = bomberheight;
            EntityBomber = new EntityBomber();
            EntityBomber.Init(speed, weight, bodyColor, additionalColor, bombs, fuelTanks, line);
            if ((_bomberWidth >= _pictureWidth) || (_bomberHeight >= _pictureHeight))
            {
                Console.WriteLine("Объект не прошел проверку");
                return false;
            }
            return true;
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            _startPosX = x;
            _startPosY = y;
            // если выходит за границы, возвращаем на форму
            if (_startPosX + _bomberWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth - _bomberWidth;
            }
            if (_startPosY + _bomberHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight - _bomberHeight;
            }
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (EntityBomber == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    if (_startPosX - EntityBomber.Step > 0)
                    {
                        _startPosX -= (int)EntityBomber.Step;
                    }
                    break;
                //вверх
                case DirectionType.Up:
                    if (_startPosY - EntityBomber.Step > 0)
                    {
                        _startPosY -= (int)EntityBomber.Step;
                    }
                    break;
                // вправо
                case DirectionType.Right:
                    if (_startPosX + EntityBomber.Step > 0)
                    {
                        _startPosX += (int)EntityBomber.Step;
                    }
                    if (_startPosX + _bomberWidth > _pictureWidth)
                    {
                        _startPosX = _pictureWidth - _bomberWidth;
                    }
                    break;
                //вниз
                case DirectionType.Down:
                    if (_startPosY + EntityBomber.Step > 0)
                    {
                        _startPosY += (int)EntityBomber.Step;
                    }
                    if (_startPosY + _bomberHeight > _pictureHeight)
                    {
                        _startPosY = _pictureHeight - _bomberHeight;
                    }
                    break;
            }
        }
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (EntityBomber == null)
            {
                return;
            }
            Pen pen = new Pen(Color.Black);
            Brush additionalBrush = new SolidBrush(EntityBomber.AdditionalColor);
            Brush bodyBrush = new SolidBrush(EntityBomber.BodyColor);
            // Бомбы
            if (EntityBomber.Bombs)
            {
                g.FillRectangle(additionalBrush, _startPosX + 15, _startPosY + 18, 18, 4);
                g.FillRectangle(additionalBrush, _startPosX + 15, _startPosY + 48, 18, 4);
            }
            // крыло 1
            GraphicsPath path = new GraphicsPath();
            path.StartFigure();
            path.AddLine(_startPosX + 20, _startPosY + 30, _startPosX + 20, _startPosY + 0);
            path.AddLine(_startPosX + 20, _startPosY + 0, _startPosX + 25, _startPosY + 0);
            path.AddLine(_startPosX + 25, _startPosY + 0, _startPosX + 30, _startPosY + 30);
            path.AddLine(_startPosX + 30, _startPosY + 30, _startPosX + 20, _startPosY + 30);
            path.CloseFigure();
            // Заливаем цветом
            g.FillPath(Brushes.Blue, path);
            // Рисуем контур линии
            // крыло 2
            g.DrawPath(pen, path);
            path.StartFigure();
            path.AddLine(_startPosX + 20, _startPosY + 40, _startPosX + 20, _startPosY + 70);
            path.AddLine(_startPosX + 20, _startPosY + 70, _startPosX + 25, _startPosY + 70);
            path.AddLine(_startPosX + 25, _startPosY + 70, _startPosX + 30, _startPosY + 40);
            path.AddLine(_startPosX + 30, _startPosY + 40, _startPosX + 20, _startPosY + 40);
            path.CloseFigure();
            // Заливаем цветом
            g.FillPath(Brushes.Blue, path);
            // Рисуем контур линии
            g.DrawPath(pen, path);
            // фюзеляж
            GraphicsPath path1 = new GraphicsPath();
            Brush brGreen = new SolidBrush(Color.Green);
            g.FillRectangle(bodyBrush, _startPosX + 10, _startPosY + 30, 50, 10);
            //нос
            path1.StartFigure();
            path1.AddLine(_startPosX + 10, _startPosY + 30, _startPosX + 0, _startPosY + 35);
            path1.AddLine(_startPosX + 0, _startPosY + 35, _startPosX + 10, _startPosY + 40);
            path1.AddLine(_startPosX + 10, _startPosY + 40, _startPosX + 10, _startPosY + 30);
            path1.CloseFigure();
            // Заливаем цветом
            Brush brOrange = new SolidBrush(Color.Orange);
            g.FillPath(brOrange, path1);
            // Рисуем контур линии
            g.DrawPath(pen, path1);
            // Горизонтальное оперение 1
            path1.StartFigure();
            path1.AddLine(_startPosX + 50, _startPosY + 30, _startPosX + 60, _startPosY + 20);
            path1.AddLine(_startPosX + 60, _startPosY + 20, _startPosX + 60, _startPosY + 30);
            path1.AddLine(_startPosX + 60, _startPosY + 30, _startPosX + 50, _startPosY + 30);
            path1.CloseFigure();
            // Заливаем цветом
            g.FillPath(brOrange, path1);
            // Рисуем контур линии
            g.DrawPath(pen, path1);
            // Горизонтальное оперение 2
            path1.StartFigure();
            path1.AddLine(_startPosX + 50, _startPosY + 40, _startPosX + 60, _startPosY + 50);
            path1.AddLine(_startPosX + 60, _startPosY + 50, _startPosX + 60, _startPosY + 40);
            path1.AddLine(_startPosX + 60, _startPosY + 40, _startPosX + 50, _startPosY + 40);
            path1.CloseFigure();
            // Заливаем цветом
            g.FillPath(brOrange, path1);
            // Рисуем контур линии
            g.DrawPath(pen, path1);
            // баки с топливом
            if (EntityBomber.FuelTanks)
            {
                g.FillRectangle(additionalBrush, _startPosX + 40, _startPosY + 28, 10, 3);
                g.FillRectangle(additionalBrush, _startPosX + 40, _startPosY + 40, 10, 3);
            }
            // линия
            if (EntityBomber.Line)
            {
                g.FillRectangle(additionalBrush, _startPosX + 10, _startPosY + 34, 50, 2);
            }
        }
    }
}
